#include <iostream>
#include <cstdio>
#include <cstring>
#include <algorithm>
#include <cassert>
using namespace std;

const int N_MAX = 100050, E_MAX = 2 * N_MAX;

int N, L, W, E;
int eadj[E_MAX], eweight[E_MAX], eprev[E_MAX], elast[N_MAX];
bool erased[E_MAX];

int front, back, q[N_MAX];
int vis[N_MAX], V = 0;
int len[N_MAX], dist[N_MAX], size[N_MAX];

pair<int, int> dist_len[N_MAX];

void add_edge(int a, int b, int weight)
{
    eadj[E] = b; eweight[E] = weight; eprev[E] = elast[a]; elast[a] = E++;
}

int bfs(int start, bool make_list)
{
    front = back = 0;
    V++;

    q[back++] = start;
    vis[start] = V;
    len[start] = dist[start] = 0;

    while (front < back)
    {
        int top = q[front++];

        for (int i = elast[top]; i != -1; i = eprev[i])
            if (!erased[i] && vis[eadj[i]] != V)
            {
                vis[eadj[i]] = V;
                len[eadj[i]] = len[top] + 1;
                dist[eadj[i]] = dist[top] + eweight[i];
                q[back++] = eadj[i];
            }
    }

    int Q = back;

    for (int i = Q - 1; i >= 0; i--)
    {
        int num = q[i];
        size[num] = 1;

        for (int j = elast[num]; j != -1; j = eprev[j])
            if (!erased[j] && len[eadj[j]] == len[num] + 1)
                size[num] += size[eadj[j]];
    }

    if (make_list)
    {
        for (int i = 0; i < Q; i++)
            dist_len[i] = make_pair(dist[q[i]], len[q[i]]);

        sort(dist_len, dist_len + Q);
    }

    return Q;
}

int center(int start)
{
    bfs(start, false);
    int pos, most = start;

    do
    {
        pos = most;
        most = -1;

        for (int i = elast[pos]; i != -1; i = eprev[i])
            if (!erased[i] && len[eadj[i]] == len[pos] + 1 && 2 * size[eadj[i]] > size[start])
            {
                most = eadj[i];
                break;
            }
    }
    while (most != -1);

    return pos;
}

int T, tree[N_MAX];

void update(int num, int change)
{
    for (int i = num + 1; i <= T; i += i & -i)
        tree[i] += change;
}

int query(int num)
{
    int sum = 0;

    for (int i = min(num + 1, T); i > 0; i -= i & -i)
        sum += tree[i];

    return sum;
}

long long pairs(int start, int len_max, int weight_max)
{
    int n = bfs(start, true);
    T = n;
    memset(tree, 0, (T + 5) * sizeof(tree[0]));
    long long total = 0;

    for (int i = n - 1, j = 0; i >= 0; i--)
    {
        while (j < n && dist_len[j].first + dist_len[i].first <= weight_max)
        {
            update(dist_len[j].second, +1);
            j++;
        }

        total += query(len_max - dist_len[i].second);

        if (2 * dist_len[i].first <= weight_max && 2 * dist_len[i].second <= len_max)
            total--;
    }

    return total;
}

long long solve(int start)
{
    long long total = 0;

    for (int i = elast[start]; i != -1; i = eprev[i])
        if (!erased[i])
        {
            erased[i ^ 1] = true;
            total -= pairs(eadj[i], L - 2, W - 2 * eweight[i]);
        }

    total += pairs(start, L, W);

    for (int i = elast[start]; i != -1; i = eprev[i])
        if (!erased[i])
            total += solve(center(eadj[i]));

    return total;
}

int main()
{
    memset(elast, -1, sizeof(elast));
    memset(erased, false, sizeof(erased));
    scanf("%d %d %d", &N, &L, &W);
    E = 0;

    for (int i = 1; i < N; i++)
    {
        int p, w;
        scanf("%d %d", &p, &w);
        p--;
        add_edge(p, i, w);
        add_edge(i, p, w);
    }

    cout << solve(center(0)) / 2 << '\n';
    return 0;
}